@@ -1,7 +1,7 @@
-# Copyright (c) 2010, OpsDojo Inc.
+# Copyright (c) 2010-2011, OpsDojo Inc.
 # All rights reserved.
 #
-# This file is subject to the New BSD License (see the LICENSE file).
+# This file is subject to the MIT License (see the LICENSE file).
 
 _m = __import__
 
@@ -130,34 +130,7 @@ def cp(src_path, dst_path, follow_links=False, recursive=True):
         logging.error('cp: execute failed: %s => %s (%s)' % (src_path, dst_path, error))
     return successful
 
-def dirs(no_class=False):
-    """Returns a reference to the directory stack used by pushd and popd.
-
-      >>> os.chdir('/tmp')
-      >>> if not pushd('/'):
-      ...     print 'Unable to pushd'
-      >>> dirs()
-      ['/tmp']
-      >>> if not popd():
-      ...     print 'Unable to popd'
-      >>> dirs()
-      []
-    """
-    # Get locals from caller
-    curframe = _m('inspect').currentframe()
-    calframe = _m('inspect').getouterframes(curframe, 2)
-    locals = calframe[1][0].f_locals
-    # Use self if caller is a method and no_class is false
-    if not no_class and 'self' in locals:
-        locals = locals['self'].__dict__
-    # Get or create directory stack variable
-    if DIRECTORY_STACK_NAME not in locals:
-        stack = locals[DIRECTORY_STACK_NAME] = []
-    else:
-        stack = locals[DIRECTORY_STACK_NAME]
-    return stack
-
-def env_get(name, default=None, type=None):
+def env_get(name, default=None, type=None, raise_exception=False):
     """Get environment variable.
 
       >>> env_get('PATH')
@@ -169,7 +142,7 @@ def env_get(name, default=None, type=None):
     """
     exists = env_has(name)
     value = _m('os').environ.get(name)
-    return normalize(value, default, type)
+    return normalize(value, default, type, raise_exception=raise_exception)
 
 def env_has(name):
     """Check if environment variable exists.
@@ -559,7 +532,7 @@ class mode(object):
         self._set_bits('_other', value)
 _ops_mode = mode
 
-def normalize(value, default=None, type=None):
+def normalize(value, default=None, type=None, raise_exception=False):
     """Convert string variables to a specified type.
 
       >>> normalize('true', type='boolean')
@@ -569,7 +542,7 @@ def normalize(value, default=None, type=None):
       >>> normalize('10.3', default=11.0)
       10.3
     """
-    NUMBER_RE = _m('re').compile('^[-+]?([0-9]+\.?[0-9]*)|([0-9]*\.?[0-9]+)$')
+    NUMBER_RE = _m('re').compile('^[-+]?(([0-9]+\.?[0-9]*)|([0-9]*\.?[0-9]+))$')
     if type is None and default is None:
         type = basestring
     elif type is None:
@@ -577,41 +550,69 @@ def normalize(value, default=None, type=None):
     if type in (basestring, 'basestring'):
         if value is not None:
             return value
-        return '' if default is None else default
+        if default is None:
+            if raise_exception:
+                raise _m('ops').exceptions.ValidationError('invalid string')
+            else:
+                return ''
     elif type in (str, 'str', 'string'):
         if value is not None:
             return value if isinstance(value, str) else str(value)
-        return '' if default is None else default
+        if default is None:
+            if raise_exception:
+                raise _m('ops').exceptions.ValidationError('invalid string')
+            else:
+                return ''
     elif type in (unicode, 'unicode'):
         if value is not None:
             return value if isinstance(value, unicode) else unicode(value)
-        return u'' if default is None else default
+        if default is None:
+            if raise_exception:
+                raise _m('ops').exceptions.ValidationError('invalid string')
+            else:
+                return u''
     elif type in (bool, 'bool', 'boolean'):
         if value is not None:
             value = value.lower().strip()
-            if value in ('1', 'true', 'yes'):
+            if value in ('1', 'true', 'yes', 'on'):
                 return True
-            elif default is None:
+            elif value in ('0', 'false', 'no', 'off'):
+                return False
+        if default is None:
+            if raise_exception:
+                raise _m('ops').exceptions.ValidationError('invalid boolean')
+            else:
                 return False
-        return False if default is None else default
     elif type in (_m('numbers').Number, 'number'):
         if value is not None:
             if value.isdigit():
                 return int(value)
-            elif value != '.' and NUMBER_RE.match(value):
+            elif NUMBER_RE.match(value):
                 return eval(value)
-        return 0 if default is None else default
+        if default is None:
+            if raise_exception:
+                raise _m('ops').exceptions.ValidationError('invalid number')
+            else:
+                return 0
     elif type in (int, 'int', 'integer'):
         try:
             return int(value)
         except Exception:
             if isinstance(value, basestring) and NUMBER_RE.match(value):
                 return int(eval(value))
-            return 0 if default is None else default
+            if default is None:
+                if raise_exception:
+                    raise _m('ops').exceptions.ValidationError('invalid number')
+                else:
+                    return 0
     elif type in (float, 'float'):
         if value is not None and NUMBER_RE.match(value):
             return float(value)
-        return 0.0 if default is None else default
+        if default is None:
+            if raise_exception:
+                raise _m('ops').exceptions.ValidationError('invalid number')
+            else:
+                return 0.0
     return default
 
 class objectify(dict):
@@ -713,91 +714,6 @@ class path(unicode):
             obj._stat = stat
         return obj
 
-def popd(no_class=False):
-    """Remove last path from the stack and make it the current working
-    directory. By default popd will look for the stack variable in self if
-    in a method and the local scope if in a function. The class behaviour can
-    be disabled by passing no_class=True.
-
-      >>> os.getcwd()
-      '/'
-      >>> if not pushd('/tmp'):
-      ...     print 'Unable to pushd'
-      >>> os.getcwd()
-      '/tmp'
-      >>> if not popd():
-      ...     print 'Unable to popd'
-      >>> os.getcwd()
-      /
-    """
-    # Get locals from caller
-    curframe = _m('inspect').currentframe()
-    calframe = _m('inspect').getouterframes(curframe, 2)
-    locals = calframe[1][0].f_locals
-    # Use self if caller is a method and no_class is false
-    if not no_class and 'self' in locals:
-        locals = locals['self'].__dict__
-    # Get or create directory stack variable
-    path = ''
-    successful = False
-    if DIRECTORY_STACK_NAME in locals:
-        stack = locals[DIRECTORY_STACK_NAME]
-        # Do popd
-        if len(stack) > 0:
-            path = stack.pop()
-            try:
-                _m('os').chdir(path)
-                successful = True
-            except OSError:
-                logging.error('popd: unable to chdir: %s' % path)
-        else:
-            logging.error('popd: stack empty')
-    else:
-        logging.error('popd: stack does not exist')
-    # Return results with path
-    return objectify({
-        '_bool': successful,
-        'path': path,
-    })
-
-def pushd(path, no_class=False):
-    """Add the current working directory to the stack and switch to the path
-    specified. By default pushd will attach the the stack variable to self if
-    in a method and the local scope if in a function. The class behaviour can
-    be disabled by passing no_class=True.
-
-    See ``pushd`` for examples.
-    """
-    # Get locals from caller
-    curframe = _m('inspect').currentframe()
-    calframe = _m('inspect').getouterframes(curframe, 2)
-    locals = calframe[1][0].f_locals
-    # Use self if caller is a method and no_class is false
-    if not no_class and 'self' in locals:
-        locals = locals['self'].__dict__
-    # Get or create directory stack variable
-    if DIRECTORY_STACK_NAME not in locals:
-        stack = locals[DIRECTORY_STACK_NAME] = []
-    else:
-        stack = locals[DIRECTORY_STACK_NAME]
-    # Do pushd
-    successful = False
-    try:
-        stack.append(_m('os').getcwd())
-        _m('os').chdir(path)
-        successful = True
-    except OSError:
-        logging.error('pushd: unable to chdir: %s' % path)
-        stack.pop()
-    # Delete variable if empty
-    if not locals[DIRECTORY_STACK_NAME]:
-        del locals[DIRECTORY_STACK_NAME]
-    # Return results with path
-    return objectify({
-        '_bool': successful,
-        'path': path,
-    })
-
 def rm(path, recursive=False):
     """Delete a specified file or directory. This function does not recursively
     delete by default.
